@@ -7,13 +7,21 @@
 ###-- Sparse ------------------------------------------------------------
 
 setMethod("cbind2", signature(x = "sparseMatrix", y = "matrix"),
-	  function(x, y, ...) cbind2(x, .Call(dense_to_Csparse, y)))
+	  function(x, y, ...) {
+              cbind2(x, .Call(R_dense_as_sparse, y, ".gC", NULL, NULL))
+          })
 setMethod("cbind2", signature(x = "matrix", y = "sparseMatrix"),
-	  function(x, y, ...) cbind2(.Call(dense_to_Csparse, x), y))
+	  function(x, y, ...) {
+              cbind2(.Call(R_dense_as_sparse, x, ".gC", NULL, NULL), y)
+          })
 setMethod("rbind2", signature(x = "sparseMatrix", y = "matrix"),
-	  function(x, y, ...) rbind2(x, .Call(dense_to_Csparse, y)))
+	  function(x, y, ...) {
+              rbind2(x, .Call(R_dense_as_sparse, y, ".gC", NULL, NULL))
+          })
 setMethod("rbind2", signature(x = "matrix", y = "sparseMatrix"),
-	  function(x, y, ...) rbind2(.Call(dense_to_Csparse, x), y))
+	  function(x, y, ...) {
+              rbind2(.Call(R_dense_as_sparse, x, ".gC", NULL, NULL), y)
+          })
 
 ## originally from ./Matrix.R : -------------------------------
 
@@ -23,7 +31,7 @@ setMethod("cbind2", signature(x = "Matrix", y = "NULL"),
 setMethod("cbind2", signature(x = "Matrix", y = "missing"),
           function(x, y, ...) x)
 setMethod("cbind2", signature(x = "NULL", y="Matrix"),
-          function(x, y, ...) x)
+          function(x, y, ...) y)
 ## using "atomicVector" not just "numeric"
 setMethod("cbind2", signature(x = "Matrix", y = "atomicVector"),
 	  function(x, y, ...) cbind2(x, matrix(y, nrow = nrow(x))))
@@ -39,7 +47,7 @@ setMethod("rbind2", signature(x = "Matrix", y = "NULL"),
 setMethod("rbind2", signature(x = "Matrix", y = "missing"),
           function(x, y, ...) x)
 setMethod("rbind2", signature(x = "NULL", y="Matrix"),
-          function(x, y, ...) x)
+          function(x, y, ...) y)
 setMethod("rbind2", signature(x = "Matrix", y = "atomicVector"),
 	  function(x, y, ...) rbind2(x, matrix(y, ncol = ncol(x))))
 setMethod("rbind2", signature(x = "atomicVector", y = "Matrix"),
@@ -72,7 +80,7 @@ setMethod("cbind2", signature(x = "denseMatrix", y = "numeric"),
 	      d <- dim(x); nr <- d[1]; nc <- d[2]
 	      y <- rep_len(y, nr) # 'silent procrustes'
 	      ## beware of (packed) triangular, symmetric, ...
-	      x <- as(x, geClass(x))
+	      x <- .dense2g(x)
 	      x@x <- c(x@x, as.double(y))
 	      x@Dim[2] <- nc + 1L
 	      if(is.character(dn <- x@Dimnames[[2]]))
@@ -84,7 +92,7 @@ setMethod("cbind2", signature(x = "numeric", y = "denseMatrix"),
 	  function(x, y, ...) {
 	      d <- dim(y); nr <- d[1]; nc <- d[2]
 	      x <- rep_len(x, nr)
-	      y <- as(y, geClass(y))
+	      y <- .dense2g(y)
 	      y@x <- c(as.double(x), y@x)
 	      y@Dim[2] <- nc + 1L
 	      if(is.character(dn <- y@Dimnames[[2]]))
@@ -94,9 +102,9 @@ setMethod("cbind2", signature(x = "numeric", y = "denseMatrix"),
 
 
 setMethod("cbind2", signature(x = "denseMatrix", y = "matrix"),
-	  function(x, y, ...) cbind2(x, as_geSimpl(y)))
+	  function(x, y, ...) cbind2(x, .m2ge(y, ".")))
 setMethod("cbind2", signature(x = "matrix", y = "denseMatrix"),
-	  function(x, y, ...) cbind2(as_geSimpl(x), y))
+	  function(x, y, ...) cbind2(.m2ge(x, "."), y))
 
 cbind2DN <- function(dnx,dny, ncx,ncy) {
     ## R and S+ are different in which names they take
@@ -104,8 +112,8 @@ cbind2DN <- function(dnx,dny, ncx,ncy) {
     rn <- if(!is.null(dnx[[1]])) dnx[[1]] else dny[[1]]
     cx <- dnx[[2]] ; cy <- dny[[2]]
     cn <- if(is.null(cx) && is.null(cy)) NULL
-    else c(if(!is.null(cx)) cx else character(ncx),
-           if(!is.null(cy)) cy else character(ncy))
+	  else c(if(!is.null(cx)) cx else character(ncx),
+		 if(!is.null(cy)) cy else character(ncy))
     list(rn, cn)
 }
 
@@ -116,8 +124,8 @@ setMethod("cbind2", signature(x = "denseMatrix", y = "denseMatrix"),
 	      ncy <- y@Dim[2]
 	      ## beware of (packed) triangular, symmetric, ...
 	      hasDN <- !is.null.DN(dnx <- dimnames(x)) | !is.null.DN(dny <- dimnames(y))
-	      x <- as(x, geClass(x))
-	      y <- as(y, geClass(y))
+	      x <- .dense2g(x)
+	      y <- .dense2g(y)
 	      xx <- c(x@x, y@x)
 	      ## be careful, e.g., if we have an 'n' and 'd'
 	      if(identical((tr <- typeof(xx)), typeof(x@x))) {
@@ -152,9 +160,9 @@ setMethod("rbind2", signature(x = "numeric", y = "denseMatrix"),
 	  })
 
 setMethod("rbind2", signature(x = "denseMatrix", y = "matrix"),
-	  function(x, y, ...) rbind2(x, as_geSimpl(y)))
+	  function(x, y, ...) rbind2(x, .m2ge(y, ".")))
 setMethod("rbind2", signature(x = "matrix", y = "denseMatrix"),
-	  function(x, y, ...) rbind2(as_geSimpl(x), y))
+	  function(x, y, ...) rbind2(.m2ge(x, "."), y))
 
 rbind2DN <- function(dnx, dny, nrx,nry) {
     if(!is.null.DN(dnx) || !is.null.DN(dny)) {
@@ -175,11 +183,8 @@ setMethod("rbind2", signature(x = "denseMatrix", y = "denseMatrix"),
 	      nry <- y@Dim[1]
 	      ## beware of (packed) triangular, symmetric, ...
 	      hasDN <- !is.null.DN(dnx <- dimnames(x)) | !is.null.DN(dny <- dimnames(y))
-	      x <- as(x, geClass(x))
-	      y <- as(y, geClass(y))
-	      ## xx <- as.vector(.Internal(rbind(-1L,
-	      ##   			      array(x@x, dim=x@Dim),
-	      ##   			      array(y@x, dim=y@Dim))))
+	      x <- .dense2g(x)
+	      y <- .dense2g(y)
 	      xx <- .Call(R_rbind2_vector, x, y)
 	      ## be careful, e.g., if we have an 'n' and 'd'
 	      if(identical((tr <- typeof(xx)), typeof(x@x))) {
@@ -199,30 +204,35 @@ setMethod("rbind2", signature(x = "denseMatrix", y = "denseMatrix"),
 
 ## For diagonalMatrix:  preserve sparseness {not always optimal, but "the law"}
 
-## hack to suppress the obnoxious dispatch ambiguity warnings:
-diag2Sp <- function(x) suppressWarnings(as(x, "CsparseMatrix"))
-
 setMethod("cbind2", signature(x = "diagonalMatrix", y = "sparseMatrix"),
-	  function(x, y, ...) cbind2(diag2Sp(x), as(y,"CsparseMatrix")))
+	  function(x, y, ...)
+              cbind2(.diag2sparse(x, ".gC"), as(y, "CsparseMatrix")))
 setMethod("cbind2", signature(x = "sparseMatrix", y = "diagonalMatrix"),
-	  function(x, y, ...) cbind2(as(x,"CsparseMatrix"), diag2Sp(y)))
+	  function(x, y, ...)
+              cbind2(as(x, "CsparseMatrix"), .diag2sparse(y, ".gC")))
 setMethod("rbind2", signature(x = "diagonalMatrix", y = "sparseMatrix"),
-	  function(x, y, ...) rbind2(diag2Sp(x), as(y,"CsparseMatrix")))
+	  function(x, y, ...)
+              rbind2(.diag2sparse(x, ".gC"), as(y, "CsparseMatrix")))
 setMethod("rbind2", signature(x = "sparseMatrix", y = "diagonalMatrix"),
-	  function(x, y, ...) rbind2(as(x,"CsparseMatrix"), diag2Sp(y)))
+	  function(x, y, ...)
+              rbind2(as(x, "CsparseMatrix"), .diag2sparse(y, ".gC")))
 
 ## in order to evade method dispatch ambiguity, but still remain "general"
 ## we use this hack instead of signature  x = "diagonalMatrix"
-for(cls in names(getClass("diagonalMatrix")@subclasses)) {
+for(cls in names(getClassDef("diagonalMatrix")@subclasses)) {
 
  setMethod("cbind2", signature(x = cls, y = "matrix"),
-	   function(x, y, ...) cbind2(diag2Sp(x), .Call(dense_to_Csparse, y)))
+	   function(x, y, ...)
+               cbind2(.diag2sparse(x, ".gC"), .m2sparse(y, ".gC")))
  setMethod("cbind2", signature(x = "matrix", y = cls),
-	   function(x, y, ...) cbind2(.Call(dense_to_Csparse, x), diag2Sp(y)))
+	   function(x, y, ...)
+               cbind2(.m2sparse(x, ".gC"), .diag2sparse(y, ".gC")))
  setMethod("rbind2", signature(x = cls, y = "matrix"),
-	   function(x, y, ...) rbind2(diag2Sp(x), .Call(dense_to_Csparse, y)))
+	   function(x, y, ...)
+               rbind2(.diag2sparse(x, ".gC"), .m2sparse(y, ".gC")))
  setMethod("rbind2", signature(x = "matrix", y = cls),
-	   function(x, y, ...) rbind2(.Call(dense_to_Csparse, x), diag2Sp(y)))
+	   function(x, y, ...)
+               rbind2(.m2sparse(x, ".gC"), .diag2sparse(y, ".gC")))
 
  ## These are already defined for "Matrix"
  ## -- repeated here for method dispatch disambiguation	 {"design-FIXME" ?}
@@ -235,7 +245,7 @@ for(cls in names(getClass("diagonalMatrix")@subclasses)) {
  setMethod("rbind2", signature(x = "atomicVector", y = cls),
 	   function(x, y, ...) rbind2(matrix(x, ncol = ncol(y)), y))
 }
-
+rm(cls)
 
 ## originally from ./dsparseMatrix.R : --------------------------------
 
@@ -243,8 +253,8 @@ for(cls in names(getClass("diagonalMatrix")@subclasses)) {
 ## ------> ../src/Csparse.c
 
 ## Fast - almost non-checking methods
-.cbind2Csp <- function(x,y) .Call(Csparse_horzcat, as_Csp2(x), as_Csp2(y))
-.rbind2Csp <- function(x,y) .Call(Csparse_vertcat, as_Csp2(x), as_Csp2(y))
+.cbind2Csp <- function(x,y) .Call(Csparse_horzcat, asCspN(x), asCspN(y))
+.rbind2Csp <- function(x,y) .Call(Csparse_vertcat, asCspN(x), asCspN(y))
 
 cbind2sparse <- function(x,y) {
     ## beware of (packed) triangular, symmetric, ...
@@ -304,28 +314,32 @@ setMethod("cbind2", signature(x = "sparseMatrix", y = "denseMatrix"),
 	  function(x, y, sparse = NA, ...) {
 	      nr <- rowCheck(x,y)
 	      if(is.na(sparse)) # result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < nr * (ncol(x)+ncol(y))
+		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		      as.double(nr) * (ncol(x)+ncol(y)) # as.double(): avoid integer overflow in '*'
 	      if(sparse) cbind2sparse(x,y) else cbind2(as(x, "denseMatrix"), y)
 	  })
 setMethod("cbind2", signature(x = "denseMatrix", y = "sparseMatrix"),
 	  function(x, y, sparse = NA, ...) {
 	      nr <- rowCheck(x,y)
 	      if(is.na(sparse)) # result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < nr * (ncol(x)+ncol(y))
+		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		      as.double(nr) * (ncol(x)+ncol(y))
 	      if(sparse) cbind2sparse(x,y) else cbind2(x, as(y, "denseMatrix"))
 	  })
 setMethod("rbind2", signature(x = "sparseMatrix", y = "denseMatrix"),
 	  function(x, y, sparse = NA, ...) {
 	      nc <- colCheck(x,y)
 	      if(is.na(sparse)) # result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < (nrow(x)+nrow(y)) * nc
+		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		      (nrow(x)+nrow(y)) * as.double(nc)
 	      if(sparse) rbind2sparse(x,y) else rbind2(as(x, "denseMatrix"), y)
 	  })
 setMethod("rbind2", signature(x = "denseMatrix", y = "sparseMatrix"),
 	  function(x, y, sparse = NA, ...) {
 	      nc <- colCheck(x,y)
 	      if(is.na(sparse)) # result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < (nrow(x)+nrow(y)) * nc
+		  sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		(nrow(x)+nrow(y)) * as.double(nc)
 	      if(sparse) rbind2sparse(x,y) else rbind2(x, as(y, "denseMatrix"))
 	  })
 
@@ -335,28 +349,32 @@ setMethod("cbind2", signature(x = "sparseMatrix", y = "denseMatrix"),
 	  function(x, y, ...) {
 	      nr <- rowCheck(x,y)
 	      ## result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < nr * (ncol(x)+ncol(y))
+	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		  as.double(nr) * (ncol(x)+ncol(y))
 	      if(sparse) cbind2sparse(x,y) else cbind2(as(x, "denseMatrix"), y)
 	  })
 setMethod("cbind2", signature(x = "denseMatrix", y = "sparseMatrix"),
 	  function(x, y, ...) {
 	      nr <- rowCheck(x,y)
 	      ## result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < nr * (ncol(x)+ncol(y))
+	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		  as.double(nr) * (ncol(x)+ncol(y))
 	      if(sparse) cbind2sparse(x,y) else cbind2(x, as(y, "denseMatrix"))
 	  })
 setMethod("rbind2", signature(x = "sparseMatrix", y = "denseMatrix"),
 	  function(x, y, ...) {
 	      nc <- colCheck(x,y)
 	      ## result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < (nrow(x)+nrow(y)) * nc
+	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		  (nrow(x)+nrow(y)) * as.double(nc)
 	      if(sparse) rbind2sparse(x,y) else rbind2(as(x, "denseMatrix"), y)
 	  })
 setMethod("rbind2", signature(x = "denseMatrix", y = "sparseMatrix"),
 	  function(x, y, ...) {
 	      nc <- colCheck(x,y)
 	      ## result is sparse if "enough zeros" <==> sparseDefault() in Matrix()
-	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 < (nrow(x)+nrow(y)) * nc
+	      sparse <- (nnzero(x,na.counted=TRUE)+nnzero(y,na.counted=TRUE)) * 2 <
+		  (nrow(x)+nrow(y)) * as.double(nc)
 	      if(sparse) rbind2sparse(x,y) else rbind2(x, as(y, "denseMatrix"))
 	  })
 }# older R -- no "sparse = NA"
@@ -409,16 +427,41 @@ setMethod("cbind2", signature(x = "numeric", y = "sparseMatrix"),
 
 }## -- no longer
 
-## Can be made very efficient
 setMethod("rbind2", signature(x = "indMatrix", y = "indMatrix"),
 	  function(x, y, ...) {
 	      dx <- x@Dim
 	      dy <- y@Dim
-	      if(dx[2] != dy[2])
-		  stop(gettextf("Matrices must have same number of columns in %s",
+              if((n <- dx[2L]) != dy[2L])
+		  stop(gettextf("number of columns of matrices must match in %s",
 				deparse(sys.call(sys.parent()))),
-		       call. = FALSE, domain=NA)
-	      new("indMatrix", Dim = c(dx[1]+dy[1], dx[2]),
-		  perm = c(x@perm,y@perm),
-		  Dimnames = rbind2DN(dimnames(x), dimnames(y), dx[1],dy[1]))
+		       call. = FALSE, domain = NA)
+              if(x@margin != 1L || y@margin != 1L)
+                  return(rbind2(as(x, "RsparseMatrix"), as(y, "RsparseMatrix")))
+              mx <- dx[1L]
+              my <- dy[1L]
+              r <- new("indMatrix")
+              r@Dim <- c(mx + my, n)
+              r@Dimnames <- rbind2DN(x@Dimnames, y@Dimnames, mx, my)
+              r@perm <- c(x@perm, y@perm)
+              r
+	  })
+
+setMethod("cbind2", signature(x = "indMatrix", y = "indMatrix"),
+	  function(x, y, ...) {
+	      dx <- x@Dim
+	      dy <- y@Dim
+              if((m <- dx[1L]) != dy[1L])
+		  stop(gettextf("number of rows of matrices must match in %s",
+				deparse(sys.call(sys.parent()))),
+		       call. = FALSE, domain = NA)
+              if(x@margin == 1L || y@margin == 1L)
+                  return(cbind2(as(x, "CsparseMatrix"), as(y, "CsparseMatrix")))
+              nx <- dx[2L]
+              ny <- dy[2L]
+              r <- new("indMatrix")
+              r@Dim <- c(m, nx + ny)
+              r@Dimnames <- cbind2DN(x@Dimnames, y@Dimnames, nx, ny)
+              r@perm <- c(x@perm, y@perm)
+              r@margin <- 2L
+              r
 	  })
