@@ -1,245 +1,49 @@
-#### All Methods in relation with the sparseVector (sub)classes
+## METHODS FOR CLASS: sparseVector (virtual)
+## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 
-## atomicVector : classUnion (logical,integer,double,....)
-setAs("atomicVector", "sparseVector",
-      function(from) {
-	  n <- length(from)# *is* integer for atomic vectors
-	  r <- new(paste0(.V.kind(from), "sparseVector"), length = n)
-	  ii <- isN0(from)
-	  r@x <- from[ii]
-	  r@i <- seq_len(n)[ii]
-	  r
-      })
-## dsparseVector: currently important, as later potentially made into d..Matrix :
-setAs("atomicVector", "dsparseVector",
-      function(from) {
-	  n <- length(from)# *is* integer for atomic vectors
-	  r <- new("dsparseVector", length = n)
-	  ii <- isN0(from)
-	  r@x <- as.numeric(from)[ii]
-	  r@i <- seq_len(n)[ii]
-	  r
-      })
+## ~~~~ COERCIONS FROM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 setAs("nsparseVector", "lsparseVector",
-      function(from) new("lsparseVector", i = from@i, length = from@length,
-			 x = rep.int(TRUE, length(from@i))))
-setAs("nsparseVector", "dsparseVector", function(from)
-      as(as(from, "lsparseVector"), "dsparseVector"))
-setAs("nsparseVector", "isparseVector", function(from)
-      as(as(from, "lsparseVector"), "isparseVector"))
-setAs("nsparseVector", "zsparseVector", function(from)
-      as(as(from, "lsparseVector"), "zsparseVector"))
-
-
-## "xsparseVector" : those with an 'x' slot (i.e., currently := not nsparse*)
-setAs("xsparseVector", "dsparseVector",
       function(from)
-      new("dsparseVector", x= as.double(from@x) , i= from@i, length= from@length))
-setAs("xsparseVector", "isparseVector",
+          new("lsparseVector", length = from@length, i = from@i,
+              x = rep.int(TRUE, length(from@i))))
+setAs("nsparseVector", "isparseVector",
       function(from)
-      new("isparseVector", x= as.integer(from@x), i= from@i, length= from@length))
-setAs("xsparseVector", "lsparseVector",
+          new("isparseVector", length = from@length, i = from@i,
+              x = rep.int(1L, length(from@i))))
+setAs("nsparseVector", "dsparseVector",
       function(from)
-      new("lsparseVector", x= as.logical(from@x), i= from@i, length= from@length))
-setAs("xsparseVector", "zsparseVector",
+          new("dsparseVector", length = from@length, i = from@i,
+              x = rep.int(1, length(from@i))))
+setAs("nsparseVector", "zsparseVector",
       function(from)
-      new("zsparseVector", x= as.complex(from@x), i= from@i, length= from@length))
+          new("zsparseVector", length = from@length, i = from@i,
+              x = rep.int(1+0i, length(from@i))))
 
-setAs("xsparseVector", "nsparseVector",
-      function(from) {
-	  if(anyNA(from@x))
-              stop("cannot coerce 'NA's to \"nsparseVector\"")
-          new("nsparseVector", i = from@i, length = from@length)
-      })
+setAs("sparseVector", "nsparseVector",
+      function(from)
+	  new("nsparseVector", length = from@length, i = from@i))
+setAs("sparseVector", "lsparseVector",
+      function(from)
+          new("lsparseVector", length = from@length, i = from@i,
+              x = as.logical(from@x)))
+setAs("sparseVector", "isparseVector",
+      function(from)
+          new("isparseVector", length = from@length, i = from@i,
+              x = as.integer(from@x)))
+setAs("sparseVector", "dsparseVector",
+      function(from)
+          new("dsparseVector", length = from@length, i = from@i,
+              x = as.double(from@x)))
+setAs("sparseVector", "zsparseVector",
+      function(from)
+          new("zsparseVector", length = from@length, i = from@i,
+              x = as.complex(from@x)))
 
-setMethod("is.na", signature(x = "nsparseVector"),
-	  function(x) new("nsparseVector", length = x@length))## all FALSE
-setMethod("is.na", signature(x = "sparseVector"),
-	  ## x is *not* "nsparse*" as that has own method
-	  function(x) new("nsparseVector", i = x@i[is.na(x@x)], length= x@length))
-
-
-if(getRversion() >= "3.1.0") {
-setMethod("anyNA", signature(x = "nsparseVector"), function(x) FALSE)
-setMethod("anyNA", signature(x = "sparseVector"), function(x) anyNA(x@x))
-}
-
-setMethod("is.infinite", signature(x = "nsparseVector"),
-	  function(x) new("nsparseVector", length = x@length))## all FALSE
-setMethod("is.infinite", signature(x = "sparseVector"),
-	  ## x is *not* "nsparse*" as that has own method
-	  function(x) new("nsparseVector", i = x@i[is.infinite(x@x)], length= x@length))
-
-setMethod("is.finite", signature(x = "nsparseVector"),
-	  function(x) rep.int(TRUE, x@length))## all TRUE
-setMethod("is.finite", signature(x = "sparseVector"),
-	  function(x)  {
-	      ## x is *not* "nsparse*" as that has own method
-	      r <- rep.int(TRUE, x@length) ## mostly TRUE
-	      r[x@i[!is.finite(x@x)]] <- FALSE
-	      r
-	  })
-
-
-sp2vec <- function(x, mode = .type.kind[.M.kindC(cl)]) {
-    ## sparseVector  ->  vector
-    cl <- class(x)
-    has.x <- any(.slotNames(cl) == "x") # cheap test for 'has x slot'
-    m.any <- (mode == "any")
-    if(m.any)
-	mode <- if(has.x) mode(x@x) else "logical"
-    r <- vector(mode, x@length)
-    r[x@i] <-
-	if(has.x) {
-	    if(m.any || is(x@x, mode)) x@x else as(x@x, mode)
-	} else TRUE
-    r
-}
-
-## so base functions calling as.vector() work too:
-## S3 dispatch works for base::as.vector(), but S4 dispatch does not:
-as.vector.sparseVector <- sp2vec
-as.array.sparseVector <- as.matrix.sparseVector <- function(x, ...) .sparseV2Mat(x)
-
-##' Construct new sparse vector , *dropping* zeros
-
-##' @param class  character, the sparseVector class
-##' @param x      numeric/logical/...:  the 'x' slot
-##' @param i      integer: index of non-zero entries
-##' @param length integer: the 'length' slot
-
-##' @return a sparseVector, with 0-dropped 'x' (and 'i')
-newSpV <- function(class, x, i, length, drop0 = TRUE, checkSort = TRUE) {
-    if(length(x) == 1 && (li <- length(i)) != 1) ## recycle x :
-	x <- rep.int(x, li)
-    if(drop0 && isTRUE(any(x0 <- x == 0))) {
-	keep <- is.na(x) | !x0
-	x <- x[keep]
-	i <- i[keep]
-    }
-    if(checkSort && is.unsorted(i)) {
-	ii <- sort.list(i)
-	x <- x[ii]
-	i <- i[ii]
-    }
-    new(class, x = x, i = i, length = length)
-}
-## a "version" of 'prev' with changed contents:
-newSpVec <- function(class, x, prev)
-    newSpV(class, x=x, i=prev@i, length=prev@length)
-
-## Exported:
-sparseVector <- function(x, i, length) {
-    newSpV(class = paste0(.V.kind(x), "sparseVector"),
-           x=x, i=i, length=length)
-}
-
-
-setAs("sparseVector", "vector", function(from) sp2vec(from))
-
-setMethod("as.vector", "sparseVector", sp2vec)
-
-setMethod("as.numeric", "sparseVector", function(x) sp2vec(x, mode = "double"))
-setMethod("as.logical", "sparseVector", function(x) sp2vec(x, mode = "logical"))
-
-setAs("sparseVector", "numeric", function(from) sp2vec(from, mode = "double"))
-setAs("sparseVector", "integer", function(from) sp2vec(from, mode = "integer"))
-setAs("sparseVector", "logical", function(from) sp2vec(from, mode = "logical"))
-
-## the "catch all remaining" method:
-setAs("ANY", "sparseVector",
-      function(from) as(as.vector(from), "sparseVector"))
-## "nsparse*" is special -- by default "lsparseVector" are produced
-setAs("ANY", "nsparseVector",
-      function(from) as(as(from, "sparseVector"),"nsparseVector"))
-
-setAs("diagonalMatrix", "sparseVector",
-      function(from) {
-	  kind <- .M.kind(from) ## currently only "l" and "d" --> have 'x'
-	  n <- nrow(from)
-          n2 <- as.double(n) * n
-	  if(n2 > .Machine$integer.max) { ## double (i, length)
-	      ii <- seq(1, by = n+1, length.out = n) ## 1-based indexing
-	  } else { # integer ok
-	      n2 <- as.integer(n2)
-	      ii <- as.integer(seq(1L, by = n+1L, length.out = n))
-	  }
-	  new(paste0(kind, "sparseVector"),
-	      length = n2, i = ii,
-	      x = if(from@diag != "U") from@x else
-		  rep.int(switch(kind, "d" = 1, "l" = TRUE, "i" = 1L, "z" = 1+0i), n))
-	 })
-
-setAs("sparseMatrix", "sparseVector",
-      function(from) as(as(from, "TsparseMatrix"), "sparseVector"))
-
-setAs("CsparseMatrix", "sparseVector", ## could go via TsparseMatrix, but this is faster:
-      function(from) {
-	  d <- dim(from)
-	  n <- prod(d) # -> numeric, no integer overflow
-	  if((int.n <- n <= .Machine$integer.max)) n <- as.integer(n)
-          cld <- getClassDef(class(from))
-	  kind <- .M.kind(from, cld)
-	  if(extends(cld, "symmetricMatrix"))
-	      from <- as(from, "generalMatrix")
-	  else if(extends(cld, "triangularMatrix") && from@diag == "U")
-	      from <- .Call(Csparse_diagU2N, from)
-          xj <- .Call(Matrix_expand_pointers, from@p)
-	  ii <- if(int.n)
-	      1L + from@i + d[1] * xj
-	  else
-	      1 + from@i + as.double(d[1]) * xj
-	  cl <- paste0(kind, "sparseVector")
-	  if(kind != "n") ## have 'x' slot
-	      new(cl, i = ii, length = n, x = from@x)
-	  else
-	      new(cl, i = ii, length = n)
-      })
-
-setAs("TsparseMatrix", "sparseVector",
-      function(from) {
-	  d <- dim(from)
-	  n <- prod(d) # -> numeric, no integer overflow
-	  if((int.n <- n <= .Machine$integer.max)) n <- as.integer(n)
-          cld <- getClassDef(class(from))
-	  kind <- .M.kind(from, cld)
-	  if(extends(cld, "symmetricMatrix"))
-	      from <- as(from, "generalMatrix")
-	  else if(extends(cld, "triangularMatrix") && from@diag == "U")
-	      from <- .Call(Tsparse_diagU2N, from)
-	  if(anyDuplicatedT(from, di = d))
-	      from <- uniqTsparse(from)
-	  ii <- if(int.n)
-	      1L + from@i + d[1] * from@j
-	  else
-	      1 + from@i + as.double(d[1]) * from@j
-	  cl <- paste0(kind, "sparseVector")
-	  if(kind != "n") ## have 'x' slot
-	      new(cl, i = ii, length = n, x = from@x)
-	  else
-	      new(cl, i = ii, length = n)
-      })
-
-
-##' <description>
-##'
-##' <details>
-## Utility -- used in `dim<-` below, but also in  Matrix(.) :
-##' @title sparseVector --> sparseMatrix constructor
-##' @param x "sparseVector" object
-##' @param nrow integer or missing, as in matrix(), see ?matrix
-##' @param ncol (ditto)
-##' @param byrow logical (see ?matrix)
-##' @param check logical indicating if it needs to be checked that 'x' is a sparseVector
-##' @param symmetric logical indicating if result must be "symmetricMatrix"
-##' @return an object inheriting from "sparseMatrix"
-##' @author Martin Maechler, May 2007 ff.
-spV2M <- function (x, nrow, ncol, byrow = FALSE, check = TRUE, symmetric = FALSE)
-{
-    cx <- class(x)
-    if(check && !extends(cx, "sparseVector"))
+spV2M <- function(x, nrow, ncol, byrow = FALSE,
+                  check = TRUE, symmetric = FALSE) {
+    if(check && !is(x, "sparseVector"))
 	stop("'x' must inherit from \"sparseVector\"")
     if(!missing(ncol)) { ncol <- as.integer(ncol)
 			 if(ncol < 0) stop("'ncol' must be >= 0") }
@@ -274,19 +78,18 @@ spV2M <- function (x, nrow, ncol, byrow = FALSE, check = TRUE, symmetric = FALSE
     }
     ## now nrow * ncol >= n  (or 'symmetric')
     ##	   ~~~~~~~~~~~~~~~~
-    cld <- getClassDef(cx)
-    kind <- .M.kindC(cld)		# "d", "n", "l", "i", "z", ...
+    kind <- .M.kind(x) # "d", "n", "l", "i", "z", ...
     has.x <- kind != "n"
     clStem <- if(symmetric) "sTMatrix" else "gTMatrix"
     ## "careful_new()" :
     cNam <- paste0(kind, clStem)
-    chngCl <- is.null(slotNames(newCl <- getClass(cNam, .Force=TRUE)))
+    chngCl <- is.null(newCl <- getClassDef(cNam))
     if(chngCl) { ## e.g. "igTMatrix" is not yet implemented
-	if(substr(cNam,1,1) == "z")
+	if(kind == "z")
 	    stop(gettextf("Class %s is not yet implemented", dQuote(cNam)),
-		 domain=NA)
+		 domain = NA)
 	## coerce to "double":
-	newCl <- getClass(paste0("d", clStem))
+	newCl <- getClassDef(paste0("d", clStem))
     }
     r <- new(newCl, Dim = c(nrow, ncol))
     ## now "compute"  the (i,j,x) slots given x@(i,x)
@@ -310,29 +113,216 @@ spV2M <- function (x, nrow, ncol, byrow = FALSE, check = TRUE, symmetric = FALSE
     r@i <- i
     if(has.x) r@x <- x
     r
-}## {spV2M}
+}
 
-.sparseV2Mat <- function(from) spV2M(from, nrow=from@length, ncol=1L, check=FALSE)
-setAs("sparseVector","Matrix", .sparseV2Mat)
-setAs("sparseVector","sparseMatrix", .sparseV2Mat)
-setAs("sparseVector","TsparseMatrix", .sparseV2Mat)
-setAs("sparseVector","CsparseMatrix",
-      function(from) .Call(Tsparse_to_Csparse, .sparseV2Mat(from), FALSE))
+.sparseV2Mat <- function(from)
+    spV2M(from, nrow = from@length, ncol = 1L, check = FALSE)
 
-## This is very similar to the 'x = "sparseMatrix"' method in ./sparseMatrix.R:
-setMethod("dim<-", signature(x = "sparseVector", value = "ANY"),
+setAs("sparseVector",        "Matrix", .sparseV2Mat)
+setAs("sparseVector",  "sparseMatrix", .sparseV2Mat)
+setAs("sparseVector", "TsparseMatrix", .sparseV2Mat)
+setAs("sparseVector", "CsparseMatrix", function(from) .T2C(.sparseV2Mat(from)))
+setAs("sparseVector", "RsparseMatrix", function(from) .T2R(.sparseV2Mat(from)))
+
+sp2vec <- function(x, mode = .type.kind[.M.kind(x)]) {
+    ## sparseVector  ->  vector
+    has.x <- .hasSlot(x, "x")## has "x" slot
+    m.any <- (mode == "any")
+    if(m.any)
+	mode <- if(has.x) mode(x@x) else "logical"
+    else if(has.x) # is.<mode>() is much faster than inherits() | is():
+        xxOk <- switch(mode,
+		       "double" = is.double(x@x),
+		       "logical" = is.logical(x@x),
+		       "integer" = is.integer(x@x),
+		       "complex" = is.complex(x@x),
+		       ## otherwise (does not happen with default 'mode'):
+		       inherits(x@x, mode))
+    r <- vector(mode, x@length)
+    r[x@i] <-
+	if(has.x) {
+	    if(m.any || xxOk) x@x else as(x@x, mode)
+	} else TRUE
+    r
+}
+
+## Need 'base' functions calling as.*() to dispatch to our S4 methods:
+as.vector.sparseVector <- sp2vec
+as.matrix.sparseVector <- function(x, ...) as.matrix.default(sp2vec(x))
+ as.array.sparseVector <- function(x, ...)  as.array.default(sp2vec(x))
+
+setAs("sparseVector", "vector",  function(from) sp2vec(from))
+setAs("sparseVector", "logical", function(from) sp2vec(from, mode = "logical"))
+setAs("sparseVector", "integer", function(from) sp2vec(from, mode = "integer"))
+setAs("sparseVector", "numeric", function(from) sp2vec(from, mode = "double"))
+
+setMethod("as.vector",  "sparseVector", sp2vec)
+setMethod("as.logical", "sparseVector", function(x) sp2vec(x, mode = "logical"))
+setMethod("as.numeric", "sparseVector", function(x) sp2vec(x, mode = "double"))
+
+
+## ~~~~ COERCIONS TO ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+setAs("ANY", "sparseVector",
+      function(from) as(as.vector(from), "sparseVector"))
+
+setAs("ANY", "nsparseVector",
+      function(from) as(as(from, "sparseVector"), "nsparseVector"))
+
+setAs("atomicVector", "sparseVector",
+      function(from) {
+	  r <- new(paste0(.V.kind(from), "sparseVector"))
+	  r@length <- length(from)
+	  r@i <- ii <- which(isN0(from))
+          r@x <- from[ii]
+	  r
+      })
+
+setAs("atomicVector", "dsparseVector",
+      function(from) {
+	  r <- new("dsparseVector")
+	  r@length <- length(from)
+          r@i <- ii <- which(isN0(from))
+	  r@x <- as.double(from)[ii]
+	  r
+      })
+
+setAs("CsparseMatrix", "sparseVector",
+      function(from) .Call(CR2spV, from))
+
+setAs("RsparseMatrix", "sparseVector",
+      function(from) .Call(CR2spV, from))
+
+setAs("TsparseMatrix", "sparseVector",
+      function(from) .Call(CR2spV, .T2C(from)))
+
+setAs("diagonalMatrix", "sparseVector",
+      function(from) {
+          n <- (d <- from@Dim)[1L]
+          nn <- prod(d)
+          kind <- .M.kind(from)
+          to <- new(paste0(kind, "sparseVector"))
+          to@length <-
+              if(nn <= .Machine$integer.max)
+                  as.integer(nn)
+              else nn
+          to@i <- indDiag(n)
+          to@x <-
+              if(from@diag == "N")
+                  from@x
+              else rep.int(switch(kind,
+                                  "l" = TRUE,
+                                  "i" = 1L,
+                                  "d" = 1,
+                                  "z" = 1+0i),
+                           n)
+          to
+      })
+
+setAs("indMatrix", "sparseVector",
+      function(from) {
+          d <- from@Dim
+          m <- d[1L]
+          n <- d[2L]
+          mn <- prod(d)
+          perm <- from@perm
+          to <- new("nsparseVector")
+          if(mn <= .Machine$integer.max) {
+              to@length <- as.integer(mn)
+              to@i <-
+                  if(from@margin == 1L)
+                      seq.int(to = 0L, by = 1L, length.out = m) + perm * m
+                  else seq.int(from = 0L, by = m, length.out = n) + perm
+          } else {
+              to@length <- mn
+              to@i <-
+                  if(from@margin == 1L)
+                      seq.int(to = 0, by = 1, length.out = m) + perm * as.double(m)
+                  else seq.int(from = 0, by = as.double(m), length.out = n) + as.double(perm)
+          }
+          to
+      })
+
+
+## ~~~~ METHODS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+##' Construct new sparse vector , *dropping* zeros
+
+##' @param class  character, the sparseVector class
+##' @param x      numeric/logical/...:  the 'x' slot -- if missing ==> "nsparseVector"
+##' @param i      integer: index of non-zero entries
+##' @param length integer: the 'length' slot
+
+##' @return a sparseVector, with 0-dropped 'x' (and 'i')
+newSpV <- function(class, x, i, length, drop0 = TRUE, checkSort = TRUE) {
+    if(has.x <- !missing(x)) {
+	if(length(x) == 1 && (li <- length(i)) != 1) ## recycle x :
+	    x <- rep.int(x, li)
+	if(drop0 && isTRUE(any(x0 <- x == 0))) {
+	    keep <- is.na(x) | !x0
+	    x <- x[keep]
+	    i <- i[keep]
+	}
+    }
+    if(checkSort && is.unsorted(i)) {
+	ii <- sort.list(i)
+	if(has.x) x <- x[ii]
+	i <- i[ii]
+    }
+    if(has.x)
+	new(class, x = x, i = i, length = length)
+    else
+	new(class,        i = i, length = length)
+}
+## a "version" of 'prev' with changed contents:
+newSpVec <- function(class, x, prev)
+    newSpV(class, x=x, i=prev@i, length=prev@length)
+
+## Exported:
+sparseVector <- function(x, i, length) {
+    newSpV(class = paste0(if(missing(x)) "n" else .V.kind(x), "sparseVector"),
+           x=x, i=i, length=length)
+}
+
+setMethod("dim<-", signature(x = "sparseVector"),
 	  function(x, value) {
-	      if(!is.numeric(value) || length(value) != 2)
-		  stop("dim(.) value must be numeric of length 2")
-	      if(length(x) != prod(value <- round(value)))
-		  stop("dimensions don't match the number of cells")
-	      spV2M(x, nrow=value[1], ncol=value[2])
+	      if(!is.numeric(value) || length(value) != 2L)
+		  stop("dimensions must be numeric of length 2")
+              if(anyNA(value))
+		  stop("dimensions cannot contain NA")
+              if(any(value < 0))
+                  stop("dimensions cannot contain negative values")
+              if(!is.integer(value)) {
+                  if(any(value > .Machine$integer.max))
+                      stop("dimensions cannot exceed 2^31-1")
+                  value <- as.integer(value)
+              }
+	      if((p <- prod(value)) != (len <- length(x)))
+		  stop(gettextf("assigned dimensions [product %.0f] do not match object length [%.0f]",
+                                p, len, domain = NA))
+	      spV2M(x, nrow = value[1L], ncol = value[2L])
 	  })
 
-
 setMethod("length", "sparseVector", function(x) x@length)
 
-setMethod("t", "sparseVector", function(x) spV2M(x, nrow=1L, ncol=x@length, check=FALSE))
+setMethod("mean", signature(x = "sparseVector"),
+	  function(x, trim = 0, na.rm = FALSE, ...) {
+              if(is.numeric(trim) && length(trim) == 1L && !is.na(trim) &&
+                 trim == 0) {
+                  ## Be fast in this special case :
+                  if(isTRUE(na.rm))
+                      ## FIXME: don't allocate !is.na(x)
+                      x <- x[!is.na(x)]
+                  sum(x) / length(x)
+	      } else {
+                  ## FIXME: don't allocate as.numeric(x); need 'sort' method
+                  warning("suboptimally using as.numeric(x) to compute trimmed mean of sparseVector 'x'")
+                  mean.default(as.numeric(x), trim = trim, na.rm = na.rm, ...)
+              }
+          })
+
+setMethod("t", "sparseVector",
+          function(x) spV2M(x, nrow = 1L, ncol = x@length, check = FALSE))
 
 setMethod("show", signature(object = "sparseVector"),
    function(object) {
@@ -397,10 +387,9 @@ intIv <- function(i, n, cl.i = getClass(class(i)))
     ## Arguments: i: index vector (numeric | logical) *OR* sparseVector
     ##		  n: array extent { ==	length(.) }
     if(missing(i))
-	return(seq_len(n))
-    ## else :
-    if(extends(cl.i, "numeric")) {
-        storage.mode(i) <- "integer"
+	seq_len(n)
+    else if(extends(cl.i, "numeric")) {
+        ## not ok, when max(i) > .Machine$integer.max !  storage.mode(i) <- "integer"
         int2i(i,n) ##-> ./Tsparse.R
     }
     else if (extends(cl.i, "logical")) {
@@ -423,13 +412,14 @@ setMethod("head", signature(x = "sparseVector"),
 	      if(n >= (nx <- x@length)) return(x)
 	      if(is.integer(x@i)) n <- as.integer(n) else stopifnot(n == round(n))
               if(n < 0) n <- max(0L, n + nx)
-	      ## now be careful to *NOT* use seq_len(n), as this be efficient for huge n
-	      ## n < x@length  now.
-	      ## As we *know* that '@i' is sorted increasingly: [x@i <= n] <==> [1:kk]
 	      x@length <- n
-	      x@i <- x@i[ii <- seq_len(which.max(x@i > n) - 1L)]
-	      if(substr(class(x), 1,1) != "n") ## be fast, ...
-		  x@x <- x@x[ii]
+	      if(length(x@i)) {
+		  ## now be careful *NOT* to use seq_len(n), as this be efficient for huge n
+		  ## As we *know* that '@i' is sorted increasingly: [x@i <= n] <==> [1:kk]
+		  x@i <- x@i[ii <- seq_len(which.max(x@i > n) - 1L)]
+		  if(.hasSlot(x, "x")) ## has.x: has "x" slot
+		      x@x <- x@x[ii]
+	      }
 	      x
 	  })
 setMethod("tail", signature(x = "sparseVector"),
@@ -438,45 +428,40 @@ setMethod("tail", signature(x = "sparseVector"),
 	      if(n >= (nx <- x@length)) return(x)
 	      if(is.integer(x@i)) n <- as.integer(n) else stopifnot(n == round(n))
 	      if(n < 0) n <- max(0L, n + nx)
-	      ## now be careful to *NOT* use seq_len(n), as this be efficient for huge n
-	      ## n < x@length  now.
-	      ## As we *know*  '@i' is sorted increasingly: [x@i > nx-n] <==> [kk:nx]
 	      x@length <- n
-	      n <- nx-n # and keep indices > n
-	      N <- length(x@i)
-	      ii <- if(any(G <- x@i > n)) which.max(G):N else FALSE
-	      x@i <- x@i[ii] - n
-	      if(substr(class(x), 1,1) != "n") ## be fast, ...
-		  x@x <- x@x[ii]
+	      if((N <- length(x@i))) {
+		  ## now be careful *NOT* to use seq_len(n) ... (see above)
+		  n <- nx-n # and keep indices > n
+		  ii <- if(any(G <- x@i > n)) which.max(G):N else FALSE
+		  x@i <- x@i[ii] - n
+		  if(.hasSlot(x, "x")) ## has.x: has "x" slot
+		      x@x <- x@x[ii]
+              }
 	      x
 	  })
 
 
 setMethod("[", signature(x = "sparseVector", i = "index"),
 	  function (x, i, j, ..., drop) {
-	      cld <- getClassDef(class(x))
-	      has.x <- !extends(cld, "nsparseVector")
+	      has.x <- .hasSlot(x, "x")## has "x" slot
 	      n <- x@length
 	      if(extends(cl.i <- getClass(class(i)), "numeric") && any(i < 0)) {
-		  ## negative indices - remain sparse --> *not* using intIv()
 		  if(any(i > 0))
 		      stop("you cannot mix negative and positive indices")
 		  if(any(z <- i == 0)) i <- i[!z]
-
-		  ## all (i < 0) :
-
-		  ## FIXME: an efficient solution would use C here
+		  ## all (i < 0), negative indices:
+		  ## want to remain sparse --> *not* using intIv()
+		  ##
+		  ## TODO: more efficient solution would use C ..
 		  i <- unique(sort(-i)) # so we need to drop the 'i's
-		  if(any(nom <- is.na(m <- match(x@i, i)))) {
-		      ## eliminate those with non-0 match
-		      x@i <- x@i[nom]
-		      if(has.x) x@x <- x@x[nom]
-		  }
-		  ii <- findInterval(x@i, i)	## subtract that :
-		  x@i <- x@i - ii
-		  x@length <- x@length - length(i)
-
-              } else {
+		  nom <- is.na(m <- match(x@i, i))
+		  ## eliminate those non-0 which do match:
+		  x@i <- x@i[nom]
+		  if(has.x) x@x <- x@x[nom]
+		  ## now all x@i "appear in 'i' but must be adjusted for the removals:
+		  x@i <- x@i - findInterval(x@i, i)
+		  x@length <- n - length(i)
+              } else { ## i >= 0  or  non-numeric 'i'
                   ii <- intIv(i, n, cl.i=cl.i)
                   m <- match(x@i, ii, nomatch = 0)
                   sel <- m > 0L
@@ -485,11 +470,15 @@ setMethod("[", signature(x = "sparseVector", i = "index"),
 		  if(any(iDup <- duplicated(ii))) {
                       i.i <- match(ii[iDup], ii)
                       jm <- lapply(i.i, function(.) which(. == m))
-                      sel <- c(which(sel), unlist(jm))
+                      if (has.x) sel <- c(which(sel), unlist(jm))
                       x@i <- c(x@i, rep.int(which(iDup), lengths(jm)))
                   }
+                  if(doSort <- is.unsorted(x@i)) {
+                      io <- order(x@i, method="radix")
+                      x@i <- x@i[io]
+                  }
                   if (has.x)
-                      x@x <- x@x[sel]
+                      x@x <- if(doSort) x@x[sel][io] else x@x[sel]
               }
 	      x
 	  })
@@ -520,12 +509,11 @@ replSPvec <- function (x, i, value)
     n <- x@length
     ii <- intIv(i, n)
     lenRepl <- length(ii)
+    if(!lenRepl) return(x)
+    ## else:  lenRepl = length(ii) > 0
     lenV <- length(value)
-    if(lenV == 0) {
-	if(lenRepl != 0)
-	    stop("nothing to replace with")
-	else return(x)
-    }
+    if(lenV == 0)
+        stop("nothing to replace with")
     ## else: lenV := length(value) > 0
     if(lenRepl %% lenV != 0)
 	stop("number of items to replace is not a multiple of replacement length")
@@ -534,12 +522,11 @@ replSPvec <- function (x, i, value)
 	ir <- lenRepl:1
 	keep <- match(ii, ii[ir]) == ir
 	ii <- ii[keep]
-	lenV <- length(value <- rep(value, length = lenRepl)[keep])
+	lenV <- length(value <- rep(value, length.out = lenRepl)[keep])
 	lenRepl <- length(ii)
     }
 
-    cld <- getClassDef(class(x))
-    has.x <- !extends(cld, "nsparseVector")
+    has.x <- .hasSlot(x, "x")## has "x" slot
     m <- match(x@i, ii, nomatch = 0)
     sel <- m > 0L
 
@@ -551,17 +538,17 @@ replSPvec <- function (x, i, value)
 		x@x <- x@x[!sel]
 	}
 	return(x)
-
     }
     ## else --	some( value != 0 ) --
     if(lenV > lenRepl)
 	stop("too many replacement values")
     else if(lenV < lenRepl)
-	value <- rep(value, length = lenRepl)
-    ## now:  length(value) == lenRepl
+	value <- rep(value, length.out = lenRepl)
+    ## now:  length(value) == lenRepl > 0
 
     v0 <- is0(value)
     ## value[1:lenRepl]:  which are structural 0 now, which not?
+    v.sp <- inherits(value, "sparseVector")
 
     if(any(sel)) {
 	## indices of non-zero entries -- WRT to subvector
@@ -569,9 +556,10 @@ replSPvec <- function (x, i, value)
 
 	## 1a) replace those that are already non-zero with new val.
 	vN0 <- !v0[iN0]
-	if(any(vN0) && has.x)
-	    x@x[sel][vN0] <- value[iN0[vN0]]
-
+	if(any(vN0) && has.x) {
+	    vs <- value[iN0[vN0]]
+	    x@x[sel][vN0] <- if(v.sp) sp2vec(vs, mode=typeof(x@x)) else vs
+	}
 	## 1b) replace non-zeros with 0 --> drop entries
 	if(any(!vN0)) {
 	    i <- which(sel)[!vN0]
@@ -579,29 +567,34 @@ replSPvec <- function (x, i, value)
 		x@x <- x@x[-i]
 	    x@i <- x@i[-i]
 	}
-	iI0 <- if(length(iN0) < lenRepl)
-	    seq_len(lenRepl)[-iN0]
+	iI0 <- if(length(iN0) < lenRepl) seq_len(lenRepl)[-iN0] # else NULL
     } else iI0 <- seq_len(lenRepl)
 
     if(length(iI0) && any(vN0 <- !v0[iI0])) {
 	## 2) add those that were structural 0 (where value != 0)
 	ij0 <- iI0[vN0]
-	x@i <- c(x@i, ii[ij0])
-	if(has.x)
-	    x@x <- c(x@x, value[ij0])
+	ii <- c(x@i, ii[ij0]) # new x@i, must be sorted:
+	iInc <- sort.list(ii)
+	x@i <- ii[iInc]
+	if(has.x) # new @x, sorted along '@i':
+	    x@x <- c(x@x, if(v.sp)
+			      sp2vec(value[ij0], mode=typeof(x@x))
+			  else value[ij0]
+		     )[iInc]
     }
     x
 }
 
 setReplaceMethod("[", signature(x = "sparseVector", i = "index", j = "missing",
-				value = "replValue"),
+				value = "replValueSp"),
 		 replSPvec)
 
 setReplaceMethod("[", signature(x = "sparseVector",
                                 i = "sparseVector", j = "missing",
-				value = "replValue"),
+				value = "replValueSp"),
                  ## BTW, the important case: 'i' a *logical* sparseVector
 		 replSPvec)
+rm(replSPvec)
 
 ## Something else:  Also allow	  x[ <sparseVector> ] <- v  e.g. for atomic x :
 
@@ -614,6 +607,8 @@ setReplaceMethod("[", signature(x = "atomicVector",
 		 callGeneric(x, i = intIv(i, x@length), value=value))
 }
 
+## MJ: unused
+if(FALSE) {
 ## a "method" for c(<(sparse)Vector>, <(sparse)Vector>):
 ## FIXME: This is not exported, nor used (nor documented)
 c2v <- function(x, y) {
@@ -660,6 +655,17 @@ sortSparseV <- function(x, decreasing = FALSE, na.last = NA) {
     .NotYetImplemented()
 }
 
+##' Uniquify sparceVectors, i.e., bring them in "regularized" from,
+##' --- similar in spirit (and action!) as  uniqTsparse(.) for "TsparseMatrix"
+##' __FIXME__ better name ??  , then export and document!  __TODO__
+uniqSpVec <- function(x) {
+    ii <- sort.list(x@i, method = "radix")
+    x@i <- x@i[ii]
+    x@x <- x@x[ii]
+    x
+}
+} ## MJ
+
 all.equal.sparseV <- function(target, current, ...)
 {
     if(!is(target, "sparseVector") || !is(current, "sparseVector")) {
@@ -672,8 +678,8 @@ all.equal.sparseV <- function(target, current, ...)
 	return(paste0("sparseVector", ": lengths (", lt, ", ", lc, ") differ"))
     }
 
-    t.has.x <- class(target)  != "nsparseVector"
-    c.has.x <- class(current) != "nsparseVector"
+    t.has.x <- .hasSlot(target,  "x")## has "x" slot
+    c.has.x <- .hasSlot(current, "x")## has "x" slot
     nz.t <- length(i.t <- target @i)
     nz.c <- length(i.c <- current@i)
     t.x <- if(t.has.x)	target@x else rep.int(TRUE, nz.t)
@@ -705,7 +711,10 @@ all.equal.sparseV <- function(target, current, ...)
     ## *and* have sorted the respective i-slot, the i-slots should match!
     stopifnot(all(i.c == i.t))
 
-    all.equal.numeric(c.x, t.x, ...)
+    if(is.logical(t.x))
+        all.equal.raw(t.x, c.x, ...)
+    else
+        all.equal.numeric(t.x, c.x, ...)
 } ## all.equal.sparseV
 
 
@@ -727,6 +736,39 @@ setMethod("all.equal", c(target = "sparseVector", current = "ANY"),
 	  all.equal(as.vector(target), current, ...))
 
 
+## S3 method for 'c' [but only for dispatch on 1st arg, hence also exported as fn]
+c.sparseVector <- function(...) {
+    svl <- lapply(list(...), as, Class = "sparseVector")
+    ## cls <- unique(unlist(lapply(svl, is)))
+    ns <- vapply(svl, slot, 1, "length")
+    if((N <- sum(ns)) < .Machine$integer.max) { # some purism ..
+	ns <- as.integer(ns)
+	N <- as.integer(N)
+    }
+    narg <- length(ns)
+    iss <- lapply(svl, slot, "i")
+    ## new 'i' slot:
+    ii <- unlist(iss) + rep(cumsum(c(0L, ns[-narg])), lengths(iss))
+    ## result must have 'x' slot if we have any
+    has.x <- any(have.x <- vapply(svl, .hasSlot, logical(1L), name = "x"))
+    if(has.x) {
+	cls <- if     (any(vapply(svl, is, NA, "zsparseVector"))) "zsparseVector"
+	       else if(any(vapply(svl, is, NA, "dsparseVector"))) "dsparseVector"
+	       else if(any(vapply(svl, is, NA, "isparseVector"))) "isparseVector"
+	       else "lsparseVector"
+	if(!(all.x <- all(have.x)))
+	    one <- if     (identical(cls, "lsparseVector")) TRUE
+		   else if(identical(cls, "isparseVector")) 1L else 1.
+	xx <- unlist(if(all.x) lapply(svl, slot, "x")
+		     else lapply(seq_len(narg), function(i) {
+			 if(have.x[[i]]) svl[[i]]@x
+			 else rep_len(one, length(iss[[i]]))
+		     }))
+	new(cls, x = xx,     i = ii, length = N)
+    } else ## no "x" slot
+	new("nsparseVector", i = ii, length = N)
+}
+
 ### rep(x, ...) -- rep() is primitive with internal default method with these args:
 ### -----------
 ### till R 2.3.1, it had  rep.default()  which we use as 'model' here.
@@ -735,7 +777,7 @@ repSpV <- function(x, times) {
     ## == rep.int(<sparseVector>, times)"
     times <- as.integer(times)# truncating as rep.default()
     n <- x@length
-    has.x <- substr(class(x), 1,1) != "n" ## fast, but hackish
+    has.x <- .hasSlot(x, "x")## has "x" slot
     ## just assign new correct slots:
     if(times <= 1) { ## be quick for {0, 1} times
         if(times < 0) stop("'times >= 0' is required")
@@ -785,15 +827,6 @@ setMethod("rep", "sparseVector",
 ##		     -------
 
 
-setMethod("solve", signature(a = "Matrix", b = "sparseVector"),
-	  function(a, b, ...) callGeneric(a, as(b, "sparseMatrix")))
-
-## the 'i' slot is 1-based *and* has no NA's:
-
-setMethod("which", "nsparseVector", function(x, arr.ind) x@i)
-setMethod("which", "lsparseVector", function(x, arr.ind) x@i[is1(x@x)])
-## and *error* for "dsparseVector", "i*", ...
-
 ##' indices of vector x[] to construct  Toeplitz matrix
 ##' FIXME: write in C, port to  R('stats' package), and use in stats::toeplitz()
 ind4toeplitz <- function(n) {
@@ -801,10 +834,21 @@ ind4toeplitz <- function(n) {
     abs(as.vector(col(A) - row(A))) + 1L
 }
 
-.toeplitz.spV <-  function(x, symmetric=TRUE, giveCsparse=TRUE) {
+.toeplitz.spV <-  function(x, symmetric=TRUE, repr = c("C","T","R"), giveCsparse) {
     ## semantically "identical" to stats::toeplitz
     n <- length(x)
-    r <- spV2M(x[ind4toeplitz(n)], n,n, symmetric=symmetric, check=FALSE)
-    if (giveCsparse) as(r, "CsparseMatrix") else r
+    r <- spV2M(x[ind4toeplitz(n)], n,n, symmetric = symmetric, check = FALSE)
+    ##   ^^^^^ returning TsparseMatrix
+    if(!missing(giveCsparse)) {
+	if(missing(repr)) {
+	    repr <- if(giveCsparse) "C" else "T"
+	    warning(gettextf("'giveCsparse' has been deprecated; setting 'repr = \"%s\"' for you",
+                             repr),
+                    domain = NA)
+	} else ## !missing(repr)
+            Matrix.msg("'giveCsparse' has been deprecated; will use 'repr' instead")
+    }
+    switch(match.arg(repr), "C" = .T2C(r), "T" = r, "R" = .T2R(r))
 }
 setMethod("toeplitz", "sparseVector", .toeplitz.spV)
+rm(.toeplitz.spV)
