@@ -1,23 +1,3 @@
-## ~~~~ BACKWARDS COMPATIBILITY ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-.Rv <- getRversion() # remove after using!
-if(.Rv < "4.0.0") {
-    deparse1 <- function (expr, collapse = " ", width.cutoff = 500L, ...)
-        paste(deparse(expr, width.cutoff, ...), collapse = collapse)
-    ...length <- function()
-        ## not equivalent to base::...length, as list(...) evaluates
-        eval(quote(length(list(...))), sys.frame(-1L))
-    tryInvokeRestart <- function(r, ...) {
-        if(!isRestart(r))
-            r <- findRestart(r)
-        if(is.null(r))
-            invisible(NULL)
-        else .Internal(.invokeRestart(r, list(...)))
-    }
-}
-rm(.Rv)
-
-
 ## ~~~~ PACKAGE ENVIRONMENTS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 ## Recording default values of Matrix.* options
@@ -30,6 +10,33 @@ rm(.Rv)
 ## ~~~~ NAMESPACE HOOKS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 .onLoad <- function(libname, pkgname) {
+    ## For backwards compatibility with earlier versions of R,
+    ## at least until x.y.z if we have Depends: R (>= x.y.z)
+    if((Rv <- getRversion()) < "4.1.3" &&
+       ## Namespace not locked yet, but being defensive here:
+       !environmentIsLocked(Mns <- parent.env(environment()))) {
+        assign("...names", envir = Mns, inherits = FALSE,
+               function() eval(quote(names(list(...))), sys.frame(-1L)))
+        if(Rv < "4.0.0") {
+            assign("deparse1", envir = Mns, inherits = FALSE,
+                   function(expr, collapse = " ", width.cutoff = 500L, ...)
+                       paste(deparse(expr, width.cutoff, ...),
+                             collapse = collapse))
+            assign("sequence.default", envir = Mns, inherits = FALSE,
+                   function(nvec, from = 1L, by = 1L, ...)
+                       unlist(.mapply(seq.int,
+                                      list(from = as.integer(from),
+                                           by = as.integer(by),
+                                           length.out = as.integer(nvec)),
+                                      NULL),
+                              recursive = FALSE, use.names = FALSE))
+            assign("tryInvokeRestart", envir = Mns, inherits = FALSE,
+                   function(r, ...)
+                       tryCatch(invokeRestart(r, ...),
+                                error = function(e) invisible(NULL)))
+        }
+    }
+
     ## verbose:
     ## logical/integer (but often supplied as double),
     ## deciding _if_ conditions are signaled
@@ -60,6 +67,14 @@ rm(.Rv)
     wDC <- as.integer(Sys.getenv("R_MATRIX_WARN_DEPRECATED_COERCE", NA))
     assign("warnDeprecatedCoerce", wDC, envir = .MatrixEnv)
 
+    ## warnSqrtDefault:
+    ## <=0 ... no conditions signaled
+    ##   1 ... persistent warning
+    ## >=2 ... persistent error
+    ##  NA ... one-time warning
+    wSD <- as.integer(Sys.getenv("R_MATRIX_WARN_SQRT_DEFAULT", NA))
+    assign("warnSqrtDefault", wDC, envir = .MatrixEnv)
+
     .Call(CHM_set_common_env, .chm_common)
     NULL
 }
@@ -67,13 +82,101 @@ rm(.Rv)
 .onUnload <- function(libpath) {
     library.dynam.unload("Matrix", libpath)
     if(!.MatrixEnv[["ambiguityNotes"]])
-	options(ambiguousMethodSelection = NULL)
+        options(ambiguousMethodSelection = NULL)
     NULL
 }
 
 
 ## ~~~~ DEPRECATED ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+..2dge <- function(from) {
+    .Deprecated(new = ".dense2g", package = "Matrix")
+    .dense2g(from, "d")
+}
+.C2nC <- function(from, isTri) {
+    .Deprecated(new = ".sparse2kind", package = "Matrix")
+    .sparse2kind(from, "n")
+}
+.T2Cmat <- function(from, isTri) {
+    .Deprecated(new = ".T2CR", package = "Matrix")
+    .T2CR(from, Csparse = TRUE)
+}
+.asmatrix <- function(x) {
+    .Deprecated(new = "as(., \"matrix\")", package = "Matrix")
+    as(x, "matrix")
+}
+.dense2sy <- function(from, ...) {
+    .Deprecated(new = ".M2sym", package = "Matrix")
+    .M2sym(from, ...)
+}
+.diag2mat <- function(from) {
+    .Deprecated(new = "as(., \"matrix\")", package = "Matrix")
+    .diag2m(from)
+}
+.diag2sT <- function(from, uplo = "U", kind = ".", drop0 = TRUE) {
+    .Deprecated(new = ".diag2sparse", package = "Matrix")
+    .diag2sparse(from, code = `substr<-`(".sT", 1L, 1L, kind),
+                 uplo = uplo, drop0 = drop0)
+}
+.diag2tT <- function(from, uplo = "U", kind = ".", drop0 = TRUE) {
+    .Deprecated(new = ".diag2sparse", package = "Matrix")
+    .diag2sparse(from, code = `substr<-`(".tT", 1L, 1L, kind),
+                 uplo = uplo, drop0 = drop0)
+}
+.dsy2dsp <- function(from) {
+    .Deprecated(new = "pack", package = "Matrix")
+    pack(from)
+}
+.dsy2mat <- function(from, keep.dimnames = TRUE) {
+    .Deprecated(new = ".dense2m", package = "Matrix")
+    to <- .dense2m(from)
+    if (!keep.dimnames)
+        dimnames(to) <- NULL
+    to
+}
+.dxC2mat <- function(from, chkUdiag) {
+    .Deprecated(new = ".sparse2m", package = "Matrix")
+    .sparse2m(from)
+}
+.m2dgC <- function(from) {
+    .Deprecated(new = ".m2sparse", package = "Matrix")
+    .m2sparse(from, "dgC")
+}
+.m2lgC <- function(from) {
+    .Deprecated(new = ".m2sparse", package = "Matrix")
+    .m2sparse(from, "lgC")
+}
+.m2ngC <- function(from) {
+    .Deprecated(new = ".m2sparse", package = "Matrix")
+    if(anyNA(from))
+        stop("attempt to coerce matrix with NA to ngCMatrix")
+    .m2sparse(from, "ngC")
+}
+.m2ngCn <- function(from, na.is.not.0 = FALSE) {
+    .Deprecated(new = ".m2sparse", package = "Matrix")
+    if(!na.is.not.0 && anyNA(from))
+        stop("attempt to coerce matrix with NA to ngCMatrix")
+    .m2sparse(from, "ngC")
+}
+.m2ngTn <- function(from, na.is.not.0 = FALSE) {
+    .Deprecated(new = ".m2sparse", package = "Matrix")
+    if(!na.is.not.0 && anyNA(from))
+        stop("attempt to coerce matrix with NA to ngCMatrix")
+    .m2sparse(from, "ngT")
+}
+.n2dgT <- function(from) {
+    .Deprecated(new = ".sparse2kind", package = "Matrix")
+    .sparse2kind(from, "d")
+}
+.nC2d <- function(from) {
+    .Deprecated(new = ".sparse2kind", package = "Matrix")
+    .sparse2kind(from, "d")
+}
+.nC2l <- function(from) {
+    .Deprecated(new = ".sparse2kind", package = "Matrix")
+    .sparse2kind(from, "l")
+}
+
 ## Utility for Matrix.DeprecatedCoerce(); see below
 .as.via.virtual <- function(Class1, Class2, from = quote(from)) {
     if(!isClassDef(Class1))
@@ -110,24 +213,30 @@ Matrix.DeprecatedCoerce <- function(Class1, Class2) {
        ((w.na <- is.na(w <- as.integer(w))) || w > 0L)) {
         cln1 <- Class1@className
         cln2 <- Class2@className
-        warning. <-
-            if(w.na && grepl("d(g.|.C)Matrix", cln2))
-                function(..., call., domain) message(..., domain = domain)
-            else if(w.na || w == 1L)
-                warning
-            else stop
-        warning.(gettextf("as(<%s>, \"%s\") is deprecated since Matrix 1.4-2; do %s instead",
-                          cln1, cln2,
-                          deparse1(.as.via.virtual(Class1, Class2, quote(.)))),
-                 call. = FALSE, domain = NA)
+
+        old <- sprintf("as(<%s>, \"%s\")", cln1, cln2)
+        new <- deparse1(.as.via.virtual(Class1, Class2, quote(.)))
+
         if(w.na)
-            options(Matrix.warnDeprecatedCoerce = 0L)
+            on.exit(options(Matrix.warnDeprecatedCoerce = 0L))
+        if(w.na && grepl("d(g.|.C)Matrix", cln2)) {
+            cond <-
+                tryCatch(.Deprecated(old = old, new = new, package = "Matrix"),
+                         deprecatedWarning = identity)
+            message(conditionMessage(cond), domain = NA)
+        } else {
+            if(!w.na && w > 1L) {
+                oop <- options(warn = 2L)
+                on.exit(options(oop))
+            }
+            .Deprecated(old = old, new = new, package = "Matrix")
+        }
     }
     invisible(NULL)
 }
 
 ## "Granular" coercions available in Matrix 1.4-1,
-## all candidates for deprecation in Matrix 1.4-2:
+## all candidates for deprecation in Matrix 1.5-0:
 if(FALSE) {
 stopifnot(packageVersion("Matrix") == "1.4.1")
 dput(lapply(grep("to=\"[dln](di|ge|tr|sy|tp|sp|[gts][CRT])Matrix\"",
@@ -135,6 +244,7 @@ dput(lapply(grep("to=\"[dln](di|ge|tr|sy|tp|sp|[gts][CRT])Matrix\"",
                  value = TRUE),
             function(s) unname(eval(str2lang(paste0("c(", s, ")"))))))
 }
+## Note:  Allow  as(*, "dpoMatrix")  to check for pos.(semi)definite-ness
 .from.to <- list(c("ddenseMatrix", "dgeMatrix"), c("ddiMatrix", "dgCMatrix"),
                  c("ddiMatrix", "dgeMatrix"), c("ddiMatrix", "dtCMatrix"),
                  c("dgCMatrix", "dgeMatrix"), c("dgCMatrix", "dgTMatrix"),
@@ -256,15 +366,113 @@ for (.f.t in .from.to) {
 }
 rm(.from.to, .f.t, .f, .t, .def.template, .def, .env)
 
+setAs("CHMfactor", "Matrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"Matrix\")",
+                      new = "expand1(., \"L\")",
+                      package = "Matrix")
+          }
+          expand1(from, "L")
+      })
+
+setAs("CHMfactor", "dMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"dMatrix\")",
+                      new = "expand1(., \"L\")",
+                      package = "Matrix")
+          }
+          expand1(from, "L")
+      })
+
+setAs("CHMfactor", "dsparseMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"dsparseMatrix\")",
+                      new = "expand1(., \"L\")",
+                      package = "Matrix")
+          }
+          expand1(from, "L")
+      })
+
+setAs("CHMfactor", "sparseMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"sparseMatrix\")",
+                      new = "expand1(., \"L\")",
+                      package = "Matrix")
+          }
+          expand1(from, "L")
+      })
+
+setAs("CHMfactor", "CsparseMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"CsparseMatrix\")",
+                      new = "expand1(., \"L\")",
+                      package = "Matrix")
+          }
+          expand1(from, "L")
+      })
+
+setAs("CHMfactor", "RsparseMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"RsparseMatrix\")",
+                      new = "as(expand1(., \"L\"), \"RsparseMatrix\")",
+                      package = "Matrix")
+          }
+          as(expand1(from, "L"), "RsparseMatrix")
+      })
+
+setAs("CHMfactor", "TsparseMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"TsparseMatrix\")",
+                      new = "as(expand1(., \"L\"), \"TsparseMatrix\")",
+                      package = "Matrix")
+          }
+          as(expand1(from, "L"), "TsparseMatrix")
+      })
+
+setAs("CHMfactor", "triangularMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"triangularMatrix\")",
+                      new = "as(expand1(., \"L\"), \"triangularMatrix\")",
+                      package = "Matrix")
+          }
+          as(expand1(from, "L"), "triangularMatrix")
+      })
+
+setAs("CHMfactor", "pMatrix",
+      function(from) {
+          if(FALSE) {
+          .Deprecated(old = "as(<CHMfactor>, \"pMatrix\")",
+                      new = "expand1(., \"P1\")",
+                      package = "Matrix")
+          }
+          expand1(from, "P1")
+      })
+
+setMethod("chol2inv", signature(x = "CHMfactor"),
+          function(x, ...) {
+              if(FALSE) {
+              .Deprecated(old = "chol2inv(<CHMfactor>)",
+                          new = "solve(.)",
+                          package = "Matrix")
+              }
+              solve(x)
+          })
+
 
 ## ~~~~ DEFUNCT ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-cBind <- function(..., deparse.level = 1) {
-    .Defunct(msg = "'cBind' is defunct; 'base::cbind' handles S4 objects since R 3.2.0")
-}
-rBind <- function(..., deparse.level = 1) {
-    .Defunct(msg = "'rBind' is defunct; 'base::rbind' handles S4 objects since R 3.2.0")
-}
+cBind <- function(..., deparse.level = 1)
+    .Defunct(new = "cbind", package = "Matrix")
+rBind <- function(..., deparse.level = 1)
+    .Defunct(msg = "rbind", package = "Matrix")
 
 
 ## ~~~~ "MISCELLANEOUS" ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
