@@ -1,11 +1,11 @@
 /** @file chm_common.c
  */
 #include "chm_common.h"
-// -> Mutils.h
 
-Rboolean isValid_Csparse(SEXP x); /* -> Csparse.c */
+/* defined in Csparse.c : */
+Rboolean isValid_Csparse(SEXP);
 
-SEXP get_SuiteSparse_version() {
+SEXP get_SuiteSparse_version(void) {
     SEXP ans = allocVector(INTSXP, 3);
     int* version = INTEGER(ans);
     SuiteSparse_version(version);
@@ -22,7 +22,7 @@ static SEXP dboundSym, grow0Sym, grow1Sym, grow2Sym, maxrankSym,
     prefer_zomplexSym, prefer_upperSym, quick_return_if_not_posdefSym,
     nmethodsSym, m0_ordSym, postorderSym;
 
-void CHM_store_common() {
+void CHM_store_common(void) {
     SEXP rho = chm_common_env;
     defineVar(dboundSym, ScalarReal(c.dbound), rho);
     defineVar(grow0Sym, ScalarReal(c.grow0), rho);
@@ -47,7 +47,7 @@ void CHM_store_common() {
     defineVar(postorderSym, ScalarLogical(c.postorder), rho);
 }
 
-void CHM_restore_common() {
+void CHM_restore_common(void) {
     SEXP rho = chm_common_env, var;
 
 #define SET_AS_FROM_FRAME(_V_, _KIND_, _SYM_)	\
@@ -150,7 +150,7 @@ static void *RallocedREAL(SEXP x)
     int lx = LENGTH(rx);
     /* We over-allocate the memory chunk so that it is never NULL. */
     /* The CHOLMOD code checks for a NULL pointer even in the length-0 case. */
-    double *ans = Memcpy((double*)R_alloc(lx + 1, sizeof(double)),
+    double *ans = Memcpy((double*) R_alloc((size_t) lx + 1, sizeof(double)),
 			 REAL(rx), lx);
     UNPROTECT(1);
     return (void*)ans;
@@ -197,15 +197,15 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
     memcpy(dest, src, sizeof(cholmod_sparse));
 
     /* R_alloc the vector storage for dest and copy the contents from src */
-    np1 = src->ncol + 1;
-    nnz = (int) cholmod_nnz(src, &c);
-    dest->p = (void*) Memcpy((int*)R_alloc(np1, sizeof(int)),
-			     (int*)(src->p), np1);
-    dest->i = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
-			     (int*)(src->i), nnz);
-    if(src->xtype)
-	dest->x = (void*) Memcpy((double*)R_alloc(nnz, sizeof(double)),
-				 (double*)(src->x), nnz);
+    np1 = (size_t) src->ncol + 1;
+    nnz = (size_t) cholmod_nnz(src, &c);
+    dest->p = (void *) Memcpy((   int *) R_alloc(np1, sizeof(int)),
+			     (   int *) (src->p), np1);
+    dest->i = (void *) Memcpy((   int *) R_alloc(nnz, sizeof(int)),
+			     (   int *) (src->i), nnz);
+    if (src->xtype)
+    dest->x = (void *) Memcpy((double *) R_alloc(nnz, sizeof(double)),
+			     (double *) (src->x), nnz);
 }
 
 /**
@@ -213,20 +213,20 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
  */
 static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
 {
-    int nnz;
+    size_t nnz;
 
     /* copy all the (non-pointer) characteristics of src to dest */
     memcpy(dest, src, sizeof(cholmod_triplet));
 
     /* R_alloc the vector storage for dest and copy the contents from src */
-    nnz = src->nnz;
-    dest->i = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
-			     (int*)(src->i), nnz);
-    dest->j = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
-			     (int*)(src->j), nnz);
-    if(src->xtype)
-	dest->x = (void*) Memcpy((double*)R_alloc(nnz, sizeof(double)),
-				 (double*)(src->x), nnz);
+    nnz = (size_t) src->nnz;
+    dest->i = (void *) Memcpy((   int *) R_alloc(nnz, sizeof(int)),
+			      (   int *) (src->i), nnz);
+    dest->j = (void *) Memcpy((   int *) R_alloc(nnz, sizeof(int)),
+			      (   int *) (src->j), nnz);
+    if (src->xtype)
+    dest->x = (void *) Memcpy((double *) R_alloc(nnz, sizeof(double)),
+			      (double *) (src->x), nnz);
 }
 
 /**
@@ -248,6 +248,9 @@ static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
  * @return ans containing pointers to the slots of x, *unless*
  *	check_Udiag and x is unitriangular.
  */
+/* AS_CHM_SP  (x) := as_cholmod_sparse((CHM_SP)alloca(sizeof(cholmod_sparse)), x, TRUE,  FALSE)
+ * AS_CHM_SP__(x) := as_cholmod_sparse((CHM_SP)alloca(sizeof(cholmod_sparse)), x, FALSE, FALSE)
+ */
 CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 			 Rboolean check_Udiag, Rboolean sort_in_place)
 {
@@ -256,9 +259,11 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 	ctype = R_check_class_etc(x, valid);
     SEXP islot = GET_SLOT(x, Matrix_iSym);
 
-    if (ctype < 0) error(_("invalid class of object to as_cholmod_sparse"));
+    if (ctype < 0)
+	error(_("invalid class of object to as_cholmod_sparse"));
     if (!isValid_Csparse(x))
 	error(_("invalid object passed to as_cholmod_sparse"));
+    
     memset(ans, 0, sizeof(cholmod_sparse)); /* zero the struct */
 
     ans->itype = CHOLMOD_INT;	/* characteristics of the system */
@@ -276,7 +281,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
      * utility, but the lengths of x and i should agree. */
     ans->nzmax = LENGTH(islot);
 				/* values depending on ctype */
-    ans->x = xpt(ctype, x);
+    ans->x     = xpt  (ctype, x);
     ans->stype = stype(ctype, x);
     ans->xtype = xtype(ctype);
 
@@ -303,7 +308,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 	}
     }
 
-    if (check_Udiag && ctype % 3 == 2 // triangular
+    if (check_Udiag && ctype % 3 == 2 /* triangular */ && ans->nrow // fails for Dim = (0,0)
 	&& (*diag_P(x) == 'U')) { /* diagU2N(.)  "in place" : */
 	double one[] = {1, 0};
 	CHM_SP eye = cholmod_speye(ans->nrow, ans->ncol, ans->xtype, &c);
@@ -328,7 +333,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
  * optionally, free a or free both a and its the pointers to its contents.
  *
  * @param a  (cholmod_sparse) matrix to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 R_Free a
  * @param uploT 0 - not triangular; > 0 upper triangular; < 0 lower
  * @param Rkind - vector type to store for a->xtype == CHOLMOD_REAL,
  *                0 - REAL; 1 - LOGICAL  [unused for other a->xtype]
@@ -363,7 +368,7 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 #define DOFREE_MAYBE							\
     if (dofree > 0)							\
 	longi ? cholmod_l_free_sparse(&a, &cl) : cholmod_free_sparse(&a, &c); \
-    else if (dofree < 0) Free(a)
+    else if (dofree < 0) R_Free(a)
 
 
     switch(a->xtype) {
@@ -573,7 +578,7 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
  * optionally, free a or free both a and its the pointers to its contents.
  *
  * @param a matrix to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 R_Free a
  * @param uploT 0 - not triangular; > 0 upper triangular; < 0 lower
  * @param Rkind - vector type to store for a->xtype == CHOLMOD_REAL,
  *                0 - REAL; 1 - LOGICAL
@@ -595,7 +600,7 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 
 #define DOFREE_MAYBE					\
     if (dofree > 0) cholmod_free_triplet(&a, &c);	\
-    else if (dofree < 0) Free(a)
+    else if (dofree < 0) R_Free(a)
 
     switch(a->xtype) {
     case CHOLMOD_PATTERN:
@@ -841,7 +846,7 @@ int R_cholmod_start(CHM_CM c)
  * optionally, free a or free both a and its pointer to its contents.
  *
  * @param a matrix to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 R_Free a
  * @param Rkind type of R matrix to be generated (special to this function)
  * @param dn   -- dimnames [list(.,.) or NULL;  __already__ transposed when transp is TRUE ]
  * @param transp Rboolean, if TRUE, the result must be a copy of  t(a), i.e., "a transposed"
@@ -859,7 +864,7 @@ SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn, Rboolean transp
 
 #define DOFREE_de_MAYBE				\
     if (dofree > 0) cholmod_free_dense(&a, &c);	\
-    else if (dofree < 0) Free(a);
+    else if (dofree < 0) R_Free(a);
 
     switch(a->xtype) {		/* determine the class of the result */
 /* CHOLMOD_PATTERN never happens because cholmod_dense can't :
@@ -950,7 +955,7 @@ SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn, Rboolean transp
  * or free both a and its pointer to its contents.
  *
  * @param a cholmod_dense structure to be converted {already REAL for original l..CMatrix}
- * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 R_Free a
  * @param dn either R_NilValue or an SEXP suitable for the Dimnames slot.
  *
  * @return SEXP containing a copy of a as a matrix object
@@ -1005,7 +1010,7 @@ SEXP chm_dense_to_matrix(CHM_DN a, int dofree, SEXP dn)
  * or free both a and its pointer to its contents.
  *
  * @param a cholmod_dense structure to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 R_Free a
  *
  * @return SEXP containing a copy of a  in the sense of  as.vector(a)
  */
@@ -1045,63 +1050,64 @@ CHM_DN numeric_as_chm_dense(CHM_DN ans, double *v, int nr, int nc)
 CHM_FR as_cholmod_factor3(CHM_FR ans, SEXP x, Rboolean do_check)
 {
     static const char *valid[] = { MATRIX_VALID_CHMfactor, ""};
-    int *type = INTEGER(GET_SLOT(x, install("type"))),
-	ctype = R_check_class_etc(x, valid);
-    SEXP tmp;
+    int ctype = R_check_class_etc(x, valid);
+    if (ctype < 0)
+	error(_("object of invalid class to 'as_cholmod_factor()'"));
+    memset(ans, 0, sizeof(cholmod_factor));
 
-    if (ctype < 0) error(_("invalid class of object to as_cholmod_factor"));
-    memset(ans, 0, sizeof(cholmod_factor)); /* zero the struct */
+    SEXP tmp = GET_SLOT(x, install("type"));
+    int *type = INTEGER(tmp);
 
-    ans->itype = CHOLMOD_INT;	/* characteristics of the system */
-    ans->dtype = CHOLMOD_DOUBLE;
-    ans->z = (void *) NULL;
-    ans->xtype = (ctype < 2) ? CHOLMOD_REAL : CHOLMOD_PATTERN;
+    ans->ordering = type[0];
+    ans->is_super = type[2];
 
-    ans->ordering = type[0];	/* unravel the type */
-    ans->is_ll = (type[1] ? 1 : 0);
-    ans->is_super = (type[2] ? 1 : 0);
-    ans->is_monotonic = (type[3] ? 1 : 0);
-				/* check for consistency */
-    if ((!(ans->is_ll)) && ans->is_super)
-	error(_("Supernodal LDL' decomposition not available"));
-    if ((!type[2]) ^ (ctype % 2))
-	error(_("Supernodal/simplicial class inconsistent with type flags"));
-				/* slots always present */
-    tmp = GET_SLOT(x, Matrix_permSym);
-    ans->minor = ans->n = LENGTH(tmp); ans->Perm = INTEGER(tmp);
-    ans->ColCount = INTEGER(GET_SLOT(x, install("colcount")));
-    ans->z = ans->x = (void *) NULL;
-    if (ctype < 2) {
-	tmp = GET_SLOT(x, Matrix_xSym);
-	ans->x = REAL(tmp);
+    tmp = GET_SLOT(x, install("colcount"));
+    ans->n = LENGTH(tmp);
+    ans->minor = ans->n;
+    ans->ColCount = INTEGER(tmp);
+
+    if (ans->ordering != CHOLMOD_NATURAL)
+	ans->Perm = INTEGER(GET_SLOT(x, Matrix_permSym));
+    else {
+	int j, n = (int) ans->n, *Perm = (int *) R_alloc(ans->n, sizeof(int));
+	for (j = 0; j < n; ++j)
+	    Perm[j] = j;
+	ans->Perm = Perm;
     }
-    if (ans->is_super) {	/* supernodal factorization */
-	ans->xsize = LENGTH(tmp);
-	ans->maxcsize = type[4]; ans->maxesize = type[5];
-	ans->i = (int*)NULL;
+
+    ans->itype = CHOLMOD_INT;
+    ans->dtype = CHOLMOD_DOUBLE;
+    if (ctype >= 2)
+	ans->xtype = CHOLMOD_PATTERN;
+    else {
+	ans->xtype = CHOLMOD_REAL;
+	ans->x = REAL(GET_SLOT(x, Matrix_xSym));
+    }
+    
+    if (ans->is_super) {
 	tmp = GET_SLOT(x, install("super"));
-	ans->nsuper = LENGTH(tmp) - 1; ans->super = INTEGER(tmp);
-	/* Move these checks to the CHMfactor_validate function */
-	if (ans->nsuper < 1)
-	    error(_("Number of supernodes must be positive when is_super is TRUE"));
-	tmp = GET_SLOT(x, install("pi"));
-	if (LENGTH(tmp) != ans->nsuper + 1)
-	    error(_("Lengths of super and pi must be equal"));
-	ans->pi = INTEGER(tmp);
-	tmp = GET_SLOT(x, install("px"));
-	if (LENGTH(tmp) != ans->nsuper + 1)
-	    error(_("Lengths of super and px must be equal"));
-	ans->px = INTEGER(tmp);
-	tmp = GET_SLOT(x, install("s"));
-	ans->ssize = LENGTH(tmp); ans->s = INTEGER(tmp);
+	ans->nsuper = LENGTH(tmp) - 1;
+	ans->super = INTEGER(tmp);
+	ans->pi = INTEGER(GET_SLOT(x, install("pi")));
+	ans->px = INTEGER(GET_SLOT(x, install("px")));
+	ans->s = INTEGER(GET_SLOT(x, install("s")));
+	ans->ssize = ((int *) ans->pi)[ans->nsuper];
+	ans->xsize = ((int *) ans->px)[ans->nsuper];
+	ans->maxcsize = type[4];
+	ans->maxesize = type[5];
+	ans->is_ll = 1;
+	ans->is_monotonic = 1;
     } else {
-	ans->nzmax = LENGTH(tmp);
 	ans->p = INTEGER(GET_SLOT(x, Matrix_pSym));
 	ans->i = INTEGER(GET_SLOT(x, Matrix_iSym));
 	ans->nz = INTEGER(GET_SLOT(x, install("nz")));
 	ans->next = INTEGER(GET_SLOT(x, install("nxt")));
 	ans->prev = INTEGER(GET_SLOT(x, install("prv")));
+	ans->nzmax = ((int *) ans->p)[ans->n];
+	ans->is_ll = type[1];
+	ans->is_monotonic = type[3];
     }
+
     if (do_check && !cholmod_check_factor(ans, &c))
 	error(_("failure in as_cholmod_factor"));
     return ans;
@@ -1131,7 +1137,7 @@ CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x) {
  * optionally, free f or free both f and its pointer to its contents.
  *
  * @param f cholmod_factor object to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 R_Free a
  *
  * @return SEXP containing a copy of a
  */
@@ -1144,7 +1150,7 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
 #define DOFREE_MAYBE					\
     if(dofree) {					\
 	if (dofree > 0) cholmod_free_factor(&f, &c);	\
-	else /* dofree < 0 */ Free(f);			\
+	else /* dofree < 0 */ R_Free(f);			\
     }
 
     if(!chm_factor_ok(f)) {
@@ -1169,15 +1175,17 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = dims[1] = f->n;
 				/* copy component of known length */
-    Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_permSym, INTSXP, f->n)),
-	   (int*)f->Perm, f->n);
-    Memcpy(INTEGER(ALLOC_SLOT(ans, install("colcount"), INTSXP, f->n)),
-	   (int*)f->ColCount, f->n);
-    type = INTEGER(ALLOC_SLOT(ans, install("type"), INTSXP, f->is_super ? 6 : 4));
+    type = INTEGER(ALLOC_SLOT(ans, install("type"), INTSXP, 6));
     type[0] = f->ordering; type[1] = f->is_ll;
     type[2] = f->is_super; type[3] = f->is_monotonic;
+    type[4] = f->maxcsize; type[5] = f->maxesize;
+    Memcpy(INTEGER(ALLOC_SLOT(ans, install("colcount"), INTSXP, f->n)),
+	   (int*)f->ColCount, f->n);
+    if (f->ordering != CHOLMOD_NATURAL) {
+	Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_permSym, INTSXP, f->n)),
+	       (int*)f->Perm, f->n);
+    }
     if (f->is_super) {
-	type[4] = f->maxcsize; type[5] = f->maxesize;
 	Memcpy(INTEGER(ALLOC_SLOT(ans, install("super"), INTSXP, f->nsuper + 1)),
 	       (int*)f->super, f->nsuper+1);
 	Memcpy(INTEGER(ALLOC_SLOT(ans, install("pi"), INTSXP, f->nsuper + 1)),
@@ -1285,21 +1293,3 @@ void chm_diagN2U(CHM_SP chx, int uploT, Rboolean do_realloc)
 	cholmod_reallocate_sparse(n_nnz, chx, &c);
     return;
 }
-
-/* Placeholders; TODO: use checks above (search "CHMfactor_validate"): */
-
-SEXP CHMfactor_validate(SEXP obj) /* placeholder */
-{
-    return ScalarLogical(1);
-}
-
-SEXP CHMsimpl_validate(SEXP obj) /* placeholder */
-{
-    return ScalarLogical(1);
-}
-
-SEXP CHMsuper_validate(SEXP obj) /* placeholder */
-{
-    return ScalarLogical(1);
-}
-
