@@ -1,8 +1,12 @@
-library(Matrix)
+## for R_DEFAULT_PACKAGES=NULL :
+library(stats)
+library(utils)
 
+library(Matrix)
 source(system.file("test-tools.R", package = "Matrix"))
 
-data(KNex) ; mm <- KNex$mm
+data(KNex, package = "Matrix")
+mm <- KNex$mm
 stopifnot(##is(mm) == c("dgCMatrix", "dMatrix", "Matrix"),
           dim(mm) == (dm <- c(1850, 712)),
           identical(dimnames(mm), list(NULL,NULL)))
@@ -68,7 +72,7 @@ stopifnot(!isTRUE(all.equal.default(m1, m2)),
           identical(mc, as(m2, "CsparseMatrix")))
 
 ### -> uniq* functions now in ../R/Auxiliaries.R
-(t2 <- system.time(um2 <- Matrix:::uniq(m1)))
+(t2 <- system.time(um2 <- uniqTsparse(m1)))
 stopifnot(identical(m2,um2))
 
 ### -> error/warning condition for solve() of a singular matrix (Barry Rowlingson)
@@ -77,7 +81,7 @@ assertError(solve(M), verbose=TRUE)## ".. computationally singular" + warning +
 assertError(solve(t(M)))
 options(warn=2) # no more warnings allowed from here
 lum <- lu(M, warnSing=FALSE)
-stopifnot(is(fLU <- M@factors $ LU, "MatrixFactorization"),
+stopifnot(is(fLU <- M@factors[["denseLU"]], "MatrixFactorization"),
           identical(lum, fLU))
 (e.lu <- expand(fLU))
 M2 <- with(e.lu, P %*% L %*% U)
@@ -89,7 +93,7 @@ tt <- try(solve(M.)) # less nice: factor is *not* cached
 M1 <- M. + 0.5*Diagonal(nrow(M.))
 luM1 <- lu(M1)
 d1 <- determinant(as(M1,"denseMatrix"))
-stopifnot(identical(luM1, M1@factors$LU),
+stopifnot(identical(luM1, M1@factors[["sparseLU~"]]),
 	  diag(luM1@L) == 1,# L is *unit*-triangular
 	  all.equal(log(-prod(diag(luM1@U))), c(d1$modulus)))
 
